<?php

/**
 * Implementation of hook_drush_command().
 */
function performance_drush_command() {
  $items = array();

  $limit = "Limit the number of records. Defaults to 25.";
  $direction = "Sort direction, defaults to descending: asc or desc.";

  $items['performance-summary'] = array(
    'callback' => 'drush_performance_summary',
    'description' => "Display the Performance Logging and Monitoring summary entries.",
    'aliases' => array('perf-sm'),
    'arguments' => array(
      'limit' => $limit,
      'orderby' => "The column name to sort on, defaults to last_access: path, last_access, bytes_max, bytes_avg, ms_max, ms_avg, [query_count_max, query_count_avg, query_timer_max, query_timer_avg] or num_accesses.",
      'direction' => $direction,
    ),
    'examples' => array(
      'performance-summary 25' => 'Retrieve last 25 entries.',
      'performance-summary 10 path' => 'Retrieve last 10 entries sorted by path, descending.',
      'performance-summary 40 bytes_avg asc' => 'Retrieve last 40 entries sorted by average size, ascending.',
    ),
  );

  $items['performance-detail'] = array(
    'callback' => 'drush_performance_details',
    'description' => "Display the Performance Logging and Monitoring detail entries.",
    'aliases' => array('perf-dt'),
    'arguments' => array(
      'limit' => $limit,
      'orderby' => "The column name to sort on, defaults to timestamp: pid, timestamp, bytes, ms, [query_count, query_timer], anon or path.",
      'direction' => $direction,
    ),
    'examples' => array(
      'performance-detail 15' => 'Retrieve last 15 entries.',
      'performance-detail 20 query_count' => 'Retrieve last 20 entries sorted by the number of queries, descending.',
      'performance-detail 35 bytes asc' => 'Retrieve last 35 entries sorted by size, ascending.',
    ),
  );

  return $items;
}

/**
 * Summary page callback.
 * Differs a little from the version in the module because drush_print_table()
 * works differently.
 */
function drush_performance_summary() {
  $args = func_get_args();
  $source = array();

  // Data from last 15 minutes by default.
  $timestamp = REQUEST_TIME - 15*60;
  // Get data from first active source.
  foreach (performance_data_stores() as $store => $data) {
    if ($data['#enabled']) {
      $source['key'] = $store;
      $source['name'] = $data['#name'];
      break;
    }
  }

  if ($source['key'] == 'db') {
    // Collect arguments.
    $orderby = 'last_access';
    $columns = array('path', 'last_access', 'bytes_max', 'bytes_avg', 'ms_max', 'ms_avg', 'num_accesses');
    if (variable_get(PERFORMANCE_QUERY_VAR, 0)) {
      $columns[] = 'query_count_max';
      $columns[] = 'query_count_avg';
      $columns[] = 'query_timer_max';
      $columns[] = 'query_timer_avg';
    }
    $arguments = drush_performance_parse_args($args, $orderby, $columns);
    // Error thrown, abort.
    if (!is_array($arguments)) {
      return $arguments;
    }

    // Get actual data.
    $data_list = drush_performance_db_get_data('performance_summary', $arguments);
  }
  else {
    $data_list = call_user_func('performance_get_data_' . $source['key']);
  }

  // Build table header.
  $header = array(
    dt('Path'),
    dt('Last access'),
    dt('# accesses'),
    dt('MB Memory (Max)'),
    dt('MB Memory (Avg)'),
    dt('ms (Max)'),
    dt('ms (Avg)'),
  );
  if (variable_get(PERFORMANCE_QUERY_VAR, 0)) {
    $header[] = dt('Query ms (Max)');
    $header[] = dt('Query ms (Avg)');
    $header[] = dt('Query Count (Max)');
    $header[] = dt('Query Count (Avg)');
  }
  $rows[] = $header;

  // Format data into table.
  foreach ($data_list as $data) {
    // Cast to object because of the DB API now returning row objects by default.
    $data = is_array($data) ? (object)$data : $data;
    $total_rows++;
    $last_max = max($last_max, $data->last_access);
    $last_min = min($last_min, $data->last_access);

    // Calculate running averages.
    $total_bytes += $data->bytes_avg;
    $total_ms += $data->ms_avg;
    $total_accesses += $data->num_accesses;

    $row_data = array();
    if ($data->num_accesses > $threshold) {
      $shown++;
      $row_data[] = check_plain($data->path);
      $row_data[] = format_date($data->last_access, 'small');
      $row_data[] = $data->num_accesses;
      $row_data[] = number_format($data->bytes_max/1024/1024, 2);
      $row_data[] = number_format($data->bytes_avg/1024/1024, 2);
      $row_data[] = number_format($data->ms_max, 1);
      $row_data[] = number_format($data->ms_avg, 1);
      if (variable_get(PERFORMANCE_QUERY_VAR, 0)) {
        $row_data[] = number_format($data->query_timer_max, 1);
        $row_data[] = number_format($data->query_timer_avg, 1);
        $row_data[] = $data->query_count_max;
        $row_data[] = $data->query_count_avg;
      }
    }
    $rows[] = $row_data;
  }

  if (!$rows) {
    $rows[] = dt('No statistics available yet.');
  }

  if ($source['key'] == 'db') {
    drush_print("\033[1m" . dt('Performance log summary from !source: !limit entries ordered by !column, !direction', array('!source' => $source['name'], '!limit' => $arguments['limit'], '!column' => $arguments['orderby'], '!direction' => $arguments['direction'])) . "\033[0m", 1);
  }
  else {
    // Arguments are currently only supported when logging to the database.
    if (is_array($args) && count($args) > 0) {
      drush_log(dt('The passed arguments only work when the data source is the Drupal database. They have been ignored!'), 'warning');
    }
    drush_print("\033[1m" . dt('Performance log summary from !source for the last 15 minutes.', array('!source' => $source['name'])) . "\033[0m", 1);
  }
  drush_print_table($rows, TRUE);
}

function drush_performance_details() {
  if (!variable_get('performance_detail', 0)) {
    return drush_set_error(dt('Detail performance log is not enabled! Go to the settings page to enable it.'));
  }

  $header = array(
    dt('#'),
    dt('Date'),
    dt('Path'),
    dt('Memory (MB)'),
    dt('ms (Total)'),
    dt('Anonymous?'),
  );
  if (variable_get(PERFORMANCE_QUERY_VAR, 0)) {
    $header[] = dt('# Queries');
    $header[] = dt('Query ms');
  }
  $rows[] = $header;

  // Collect arguments.
  $args = func_get_args();
  $orderby = 'timestamp';
  $columns = array('pid', 'timestamp', 'bytes', 'ms', 'anon', 'path');
  if (variable_get(PERFORMANCE_QUERY_VAR, 0)) {
    $columns[] = 'query_count';
    $columns[] = 'query_timer';
  }
  $arguments = drush_performance_parse_args($args, $orderby, $columns);

  // Error thrown, abort.
  if (!is_array($arguments)) {
    return $arguments;
  }

  $data_list = drush_performance_db_get_data('performance_detail', $arguments);
  foreach ($data_list as $data) {
    // Cast to object because of the DB API now returning row objects by default.
    $data = is_array($data) ? (object)$data : $data;
    $row_data = array();
    $row_data[] = $data->pid;
    $row_data[] = format_date($data->timestamp, 'small');
    $row_data[] = check_plain($data->path);
    $row_data[] = number_format($data->bytes/1024/1024, 2);
    $row_data[] = $data->ms;
    $row_data[] = ($data->anon) ? t('Yes') : t('No');

    if (variable_get(PERFORMANCE_QUERY_VAR, 0)) {
      $row_data[] = $data->query_count;
      $row_data[] = $data->query_timer;
    }

    $rows[] = $row_data;
  }

  drush_print("\033[1m" . dt('Performance log details: !limit entries ordered by !column, !direction', array('!limit' => $arguments['limit'], '!column' => $arguments['orderby'], '!direction' => $arguments['direction'])) . "\033[0m", 1);
  drush_print_table($rows, TRUE);
}

/**
 * Helper function to parse the arguments which are the same for both commands.
 * Only columns differ.
 */
function drush_performance_parse_args($args, $orderby, $columns) {
  $arguments = array();

  // Set limit from arguments or fall back to default.
  $arguments['limit'] = 25;
  if (isset($args[0]) && is_numeric($args[0])) {
    $arguments['limit'] = $args[0];
  }
  
  // Order by column name.
  $arguments['orderby'] = $orderby;
  if (isset($args[1])) {
    if (in_array($args[1], $columns)) {
      $arguments['orderby'] = $args[1];
    }
    // Let the user know what the options are.
    else {
      return drush_set_error(dt('Unknown column name. Possible values are: !columns', array('!columns' => implode(', ', $columns))));
    }
  }

  // Sort direction.
  $arguments['direction'] = 'desc';
  $options = array('asc', 'desc');
  if (isset($args[2])) {
    if (in_array($args[2], $options)) {
      $arguments['direction'] = $args[2];
    }
    // Let the user know what the options are.
    else {
      return drush_set_error(dt('Unknown sort direction. Possible values are: !options', array('!options' => implode(', ', $options))));
    }
  }
  
  return $arguments;
}

/**
 * Drupal version independant variation of performance_db_get_data() for easy
 * maintenance of the drush commands.
 */
function drush_performance_db_get_data($table, $arguments) {
  $data_list = array();

  $result = drush_db_select($table, '*', NULL, NULL, 0, $arguments['limit'], $arguments['orderby'], $arguments['direction']);
  if (drush_drupal_major_version() >= 7) {
    foreach ($result as $row) {
      $data_list[] = $row;
    }
  }
  else {
    while ($row = db_fetch_object($result)) {
      $data_list[] = $row;
    }
  }

  return $data_list;
}
